@@ -26,9 +26,9 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 //! used to implement the `From` trait. Each dependency can be individually
 //! toggled with a feature named `from-{dependency}`. Current features are:
 //! * `from-reqwest`
-//! * `from-serde_json`
 //! * `from-tokio`
 use serde::Deserialize;
+use serde_json::Value;
 use std::fmt::{Display, Formatter};
 use thiserror::Error as ThisError;
 
@@ -40,6 +40,8 @@ pub struct ApiError {
     pub code: String,
     /// Error message
     pub text: String,
+    /// Extra data
+    pub data: Option<Value>,
 }
 
 impl Display for ApiError {
@@ -63,7 +65,6 @@ pub enum Error {
     #[error("Invalid header value")]
     InvalidHeaderValue(#[from] reqwest::header::InvalidHeaderValue),
     /// Error when decoding the JSON response from the API
-    #[cfg(feature = "from-serde_json")]
     #[error("JSON error")]
     JsonError(#[from] serde_json::Error),
     /// Error if unable to get request concurrency lock
@@ -103,15 +104,21 @@ pub enum Error {
     WrongPassword,
 
     /* Page-related issues */
+    /// When {{nobots}} matches
+    #[error("{{nobots}} prevents editing this page")]
+    Nobots,
     /// Page does not exist
     #[error("Page does not exist: {0}")]
     PageDoesNotExist(String),
     /// Page is protected
-    #[error("Page [[{0}]] is protected: {1}")]
-    ProtectedPage(String, String),
+    #[error("Page is protected")]
+    ProtectedPage,
     /// Edit conflict
     #[error("Edit conflict on {0}")]
     EditConflict(String),
+    /// Tripped the spam filter (aka SpamBlacklist)
+    #[error("{info}")]
+    SpamFilter { info: String, matches: Vec<String> },
 
     /* Catchall/generic issues */
     #[error("{info}, retry-after: {retry_after}")]
@@ -128,10 +135,20 @@ pub enum Error {
 impl Error {
     /// Whether the issue is related to saving a page
     pub fn is_save_error(&self) -> bool {
-        matches!(self, Error::ProtectedPage(_, _) | Error::EditConflict(_))
+        matches!(
+            self,
+            Error::ProtectedPage
+                | Error::EditConflict(_)
+                | Error::SpamFilter { .. }
+        )
     }
 }
 
+#[derive(Deserialize)]
+struct SpamFilterData {
+    matches: Vec<String>,
+}
+
 impl From<ApiError> for Error {
     fn from(apierr: ApiError) -> Self {
         match apierr.code.as_str() {
@@ -145,6 +162,24 @@ impl From<ApiError> for Error {
                 // to 0
                 retry_after: 0,
             },
+            "protectedpage" => Self::ProtectedPage,
+            "spamblacklist" => {
+                let matches = if let Some(data) = apierr.data {
+                    match serde_json::from_value::<SpamFilterData>(
+                        data["spamblacklist"].clone(),
+                    ) {
+                        Ok(data) => data.matches,
+                        // Not worth raising an error over this
+                        Err(_) => vec![],
+                    }
+                } else {
+                    vec![]
+                };
+                Self::SpamFilter {
+                    info: apierr.text,
+                    matches,
+                }
+            }
             "wrongpassword" => Self::WrongPassword,
             _ => Self::ApiError(apierr),
         }
@@ -166,6 +201,7 @@ mod tests {
         let apierr = ApiError {
             code: "assertbotfailed".to_string(),
             text: "Something something".to_string(),
+            data: None,
         };
         let err = Error::from(apierr);
         if let Error::NotLoggedInAsBot = err {
@@ -180,10 +216,32 @@ mod tests {
         let apierr = ApiError {
             code: "errorcode".to_string(),
             text: "Some description".to_string(),
+            data: None,
         };
         assert_eq!(
             &apierr.to_string(),
             "API error: (code: errorcode): Some description"
         );
     }
+
+    #[test]
+    fn test_spamfilter() {
+        let apierr = ApiError {
+            code: "spamblacklist".to_string(),
+            text: "blah blah".to_string(),
+            data: Some(serde_json::json!({
+                "spamblacklist": {
+                    "matches": [
+                        "example.org"
+                    ]
+                }
+            }))
+        };
+        let err = Error::from(apierr);
+        if let Error::SpamFilter { matches, .. } = err {
+            assert_eq!(matches, vec!["example.org".to_string()]);
+        } else {
+            panic!("Unexpected error: {:?}", err);
+        }
+    }
 }
